@@ -17,7 +17,12 @@
 #undef XSIMD_BATCH_FLOAT_SIZE
 #undef XSIMD_BATCH_DOUBLE_SIZE
 
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX_VERSION
+#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX512_VERSION
+#define XSIMD_BATCH_INT32_SIZE 16
+#define XSIMD_BATCH_INT64_SIZE 8
+#define XSIMD_BATCH_FLOAT_SIZE 16
+#define XSIMD_BATCH_DOUBLE_SIZE 8
+#elif XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX_VERSION
 #define XSIMD_BATCH_INT32_SIZE 8
 #define XSIMD_BATCH_INT64_SIZE 4
 #define XSIMD_BATCH_FLOAT_SIZE 8
@@ -109,7 +114,40 @@ namespace xsimd
         using type = float;
         static constexpr size_t size = simd_traits<type>::size;
     };
+
+    template <>
+    struct simd_traits<std::complex<float>>
+    {
+        using type = batch<std::complex<float>, XSIMD_BATCH_FLOAT_SIZE>;
+        using bool_type = simd_batch_traits<type>::batch_bool_type;
+        static constexpr size_t size = type::size;
+    };
+
+    template <>
+    struct revert_simd_traits<batch<std::complex<float>, XSIMD_BATCH_FLOAT_SIZE>>
+    {
+        using type = std::complex<float>;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+
+#ifdef XSIMD_ENABLE_XTL_COMPLEX
+    template <bool i3ec>
+    struct simd_traits<xtl::xcomplex<float, float, i3ec>>
+    {
+        using type = batch<xtl::xcomplex<float, float, i3ec>, XSIMD_BATCH_FLOAT_SIZE>;
+        using bool_type = typename simd_batch_traits<type>::batch_bool_type;
+        static constexpr size_t size = type::size;
+    };
+
+    template <bool i3ec>
+    struct revert_simd_traits<batch<xtl::xcomplex<float, float, i3ec>, XSIMD_BATCH_FLOAT_SIZE>>
+    {
+        using type = xtl::xcomplex<float, float, i3ec>;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
 #endif
+#endif
+
 #ifdef XSIMD_BATCH_DOUBLE_SIZE
     template <>
     struct simd_traits<double>
@@ -125,6 +163,38 @@ namespace xsimd
         using type = double;
         static constexpr size_t size = simd_traits<type>::size;
     };
+
+    template <>
+    struct simd_traits<std::complex<double>>
+    {
+        using type = batch<std::complex<double>, XSIMD_BATCH_DOUBLE_SIZE>;
+        using bool_type = simd_batch_traits<type>::batch_bool_type;
+        static constexpr size_t size = type::size;
+    };
+
+    template <>
+    struct revert_simd_traits<batch<std::complex<double>, XSIMD_BATCH_DOUBLE_SIZE>>
+    {
+        using type = std::complex<double>;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+
+#ifdef XSIMD_ENABLE_XTL_COMPLEX
+    template <bool i3ec>
+    struct simd_traits<xtl::xcomplex<double, double, i3ec>>
+    {
+        using type = batch<xtl::xcomplex<double, double, i3ec>, XSIMD_BATCH_DOUBLE_SIZE>;
+        using bool_type = typename simd_batch_traits<type>::batch_bool_type;
+        static constexpr size_t size = type::size;
+    };
+
+    template <bool i3ec>
+    struct revert_simd_traits<batch<xtl::xcomplex<double, double, i3ec>, XSIMD_BATCH_DOUBLE_SIZE>>
+    {
+        using type = xtl::xcomplex<double, double, i3ec>;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+#endif
 #endif
 
     /********************
@@ -141,33 +211,36 @@ namespace xsimd
                 std::is_same<T1, float>::value ||
                 std::is_same<T1, double>::value ||
                 std::is_same<T1, int64_t>::value ||
-                std::is_same<T1, int32_t>::value;
+                std::is_same<T1, int32_t>::value ||
+                std::is_same<T1, char>::value ||
+                std::is_same<T1, unsigned char>::value ||
+                (detail::is_complex<T1>::value && detail::is_complex<T2>::value);
         };
 
         template <class T1, class T2>
         struct simd_return_type_impl
+            : std::enable_if<simd_condition<T1, T2>::value, simd_type<T2>>
         {
-            using type = typename std::enable_if<simd_condition<T1, T2>::value, simd_type<T2>>::type;
         };
 
+        // Do not rely on simd_return_type_impl<T1, T2> becasue N2 can be different from
+        // simd_type<T2>::size
         template <class T1, std::size_t N1, class T2, std::size_t N2>
         struct simd_return_type_impl<batch<T1, N1>, batch<T2, N2>>
+            : std::enable_if<simd_condition<T1, T2>::value, batch<T2, N2>>
         {
-            // Do not rely on simd_return_type_impl<T1, T2> becasue N2 can be different from
-            // simd_type<T2>::size
-            using type = typename std::enable_if<simd_condition<T1, T2>::value, batch<T2, N2>>::type;
         };
 
         template <class T1, std::size_t N1, class T2, std::size_t N2>
         struct simd_return_type_impl<batch<T1, N1>, batch_bool<T2, N2>>
+            : std::enable_if<simd_condition<T1, T2>::value, batch_bool<T2, N2>>
         {
-            using type = typename std::enable_if<simd_condition<T1, T2>::value, batch_bool<T2, N2>>::type;
         };
 
         template <class T1, std::size_t N1, class T2, std::size_t N2>
         struct simd_return_type_impl<batch_bool<T1, N1>, batch_bool<T2, N2>>
+            : std::enable_if<simd_condition<T1, T2>::value, batch_bool<T2, N2>>
         {
-            using type = typename std::enable_if<simd_condition<T1, T2>::value, batch_bool<T2, N2>>::type;
         };
     }
 
